@@ -8,6 +8,7 @@ import sys
 
 import bokeh
 import distributed.bokeh
+from ..utils import ignoring
 
 dirname = os.path.dirname(distributed.__file__)
 paths = [os.path.join(dirname, 'bokeh', name)
@@ -20,17 +21,26 @@ logger = logging.getLogger(__file__)
 
 class BokehWebInterface(object):
     def __init__(self, host='127.0.0.1', http_port=9786, tcp_port=8786,
-                 bokeh_port=8787, bokeh_whitelist=[], log_level='info', show=False):
+                 bokeh_port=8787, bokeh_whitelist=[], log_level='info',
+                 show=False, prefix=None, use_xheaders=False):
         self.port = bokeh_port
         ip = socket.gethostbyname(host)
 
-        hosts = ['%s:%d' % (h, bokeh_port) for h in
-                 ['localhost',
-                  '127.0.0.1',
-                  ip, socket.gethostbyname(ip),
-                  socket.gethostname(),
-                  socket.gethostbyname(socket.gethostname()),
-                  host] + list(map(str, bokeh_whitelist))]
+        hosts = ['localhost',
+                 '127.0.0.1',
+                 ip,
+                 host]
+
+        with ignoring(Exception):
+            hosts.append(socket.gethostbyname(ip))
+        with ignoring(Exception):
+            hosts.append(socket.gethostbyname(socket.gethostname()))
+
+        hosts = ['%s:%d' % (h, bokeh_port) for h in hosts]
+
+        hosts.append("*")
+
+        hosts.extend(map(str, bokeh_whitelist))
 
         args = ([binname, 'serve'] + paths +
                 ['--log-level', 'warning',
@@ -38,9 +48,16 @@ class BokehWebInterface(object):
                  '--unused-session-lifetime=1',
                  '--port', str(bokeh_port)] +
                  sum([['--host', h] for h in hosts], []))
+
+        if prefix:
+            args.extend(['--prefix', prefix])
+
         if show:
             args.append('--show')
 
+        if use_xheaders:
+            args.append('--use-xheaders')
+
         if log_level in ('debug', 'info', 'warning', 'error', 'critical'):
             args.extend(['--log-level', log_level])
 
