@@ -250,7 +250,7 @@ class Executor(object):
     distributed.scheduler.Scheduler: Internal scheduler
     """
     def __init__(self, address=None, start=True, loop=None, timeout=3,
-                 set_as_default=False):
+                 set_as_default=True):
         self.futures = dict()
         self.refcount = defaultdict(lambda: 0)
         self._should_close_loop = loop is None and start
@@ -266,6 +266,8 @@ class Executor(object):
         if set_as_default:
             self._previous_get = _globals.get('get')
             dask.set_options(get=self.get)
+            self._previous_shuffle = _globals.get('shuffle')
+            dask.set_options(shuffle='tasks')
 
         if start:
             self.start(timeout=timeout)
@@ -444,7 +446,7 @@ class Executor(object):
         with ignoring(AttributeError):
             yield self.scheduler_stream.close(ignore_closed=True)
         with ignoring(AttributeError):
-            self.scheduler.close_streams()
+            self.scheduler.close_rpc()
 
     def shutdown(self, timeout=10):
         """ Send shutdown signal and wait until scheduler terminates """
@@ -456,13 +458,15 @@ class Executor(object):
                                    {'op': 'close-stream'})
             sync(self.loop, self.scheduler_stream.close)
         with ignoring(AttributeError):
-            self.scheduler.close_streams()
+            self.scheduler.close_rpc()
         if self._should_close_loop:
             sync(self.loop, self.loop.stop)
             self.loop.close(all_fds=True)
             self._loop_thread.join(timeout=timeout)
         with ignoring(AttributeError):
             dask.set_options(get=self._previous_get)
+        with ignoring(AttributeError):
+            dask.set_options(shuffle=self._previous_shuffle)
         if _global_executor[0] is self:
             _global_executor[0] = None
         if self.get == _globals.get('get'):
@@ -1020,7 +1024,8 @@ class Executor(object):
             result = 'OK', result
         raise gen.Return(result)
 
-    def get(self, dsk, keys, restrictions=None, loose_restrictions=None):
+    def get(self, dsk, keys, restrictions=None, loose_restrictions=None,
+            **kwargs):
         """ Compute dask graph
 
         Parameters
@@ -1497,6 +1502,58 @@ class Executor(object):
     def futures_of(self, futures):
         return futures_of(futures, executor=self)
 
+    @gen.coroutine
+    def _start_ipython(self, workers):
+        if workers is None:
+            workers = yield self.scheduler.ncores()
+
+        responses = yield self.scheduler.broadcast(
+            msg=dict(op='start_ipython'), workers=workers,
+        )
+        raise gen.Return((workers, responses))
+
+    def start_ipython(self, workers=None, magic_names=False, qtconsole=False, qtconsole_args=None):
+        """ Start IPython kernels on workers
+
+        Parameters
+        ----------
+        workers: list (optional)
+            A list of worker addresses, defaults to all
+
+        magic_names: str or list(str) (optional)
+            If defined, register IPython magics with these names for
+            executing code on the workers.
+
+        qtconsole: bool (optional)
+            If True, launch a Jupyter QtConsole connected to the worker(s).
+
+        Returns
+        -------
+        iter_connection_info: list
+            List of connection_info dicts containing info necessary
+            to connect Jupyter clients to the workers.
+        """
+
+        if magic_names and isinstance(magic_names, six.string_types):
+            magic_names = [magic_names]
+        if isinstance(workers, six.string_types):
+            workers = [workers]
+
+        (workers, info_dict) = sync(self.loop, self._start_ipython, workers)
+        if magic_names:
+            from ._ipython_utils import register_worker_magic
+            for worker, magic_name in zip(workers, magic_names):
+                connection_info = info_dict[worker]
+                register_worker_magic(connection_info, magic_name)
+        if qtconsole:
+            from ._ipython_utils import connect_qtconsole
+            for worker, connection_info in info_dict.items():
+                connect_qtconsole(connection_info,
+                                  name='dask-' + worker.replace(':','-'),
+                                  extra_args=qtconsole_args,
+                )
+        return info_dict
+
 
 class CompatibleExecutor(Executor):
     """ A concurrent.futures-compatible Executor
