@@ -11,6 +11,8 @@ from threading import Thread
 from time import sleep, time
 import traceback
 
+
+import mock
 import pytest
 from toolz import (identity, isdistinct, first, concat, pluck, valmap,
         partition_all)
@@ -307,6 +309,15 @@ def test_get_sync(loop):
             assert e.get({'x': (inc, 1)}, 'x') == 2
 
 
+def test_get_sync_optimize_grpah_passes_through(loop):
+    import dask.bag as db
+    import dask
+    bag = db.range(10, npartitions=3).map(inc)
+    with cluster() as (s, [a, b]):
+        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            dask.compute(bag.sum(), optimize_graph=False, get=e.get)
+
+
 def test_submit_errors(loop):
     def f(a, b, c):
         pass
@@ -2992,18 +3003,28 @@ def test_cancel_stress_sync(loop):
 def test_default_get(loop):
     with cluster() as (s, [a, b]):
         pre_get = _globals.get('get')
+        pre_shuffle = _globals.get('shuffle')
         with Executor(('127.0.0.1', s['port']), loop=loop, set_as_default=True) as e:
             assert _globals['get'] == e.get
+            assert _globals['shuffle'] == 'tasks'
+
         assert _globals['get'] is pre_get
+        assert _globals['shuffle'] == pre_shuffle
 
         e = Executor(('127.0.0.1', s['port']), loop=loop, set_as_default=False)
         assert _globals['get'] is pre_get
+        assert _globals['shuffle'] == pre_shuffle
         e.shutdown()
 
         e = Executor(('127.0.0.1', s['port']), loop=loop, set_as_default=True)
+        assert _globals['shuffle'] == 'tasks'
         assert _globals['get'] == e.get
         e.shutdown()
         assert _globals['get'] is pre_get
+        assert _globals['shuffle'] == pre_shuffle
+
+        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            assert _globals['get'] == e.get
 
 
 @gen_cluster(executor=True)
@@ -3168,3 +3189,79 @@ def test_as_completed_list(loop):
             seq = e.map(inc, iter(range(5)))
             seq2 = list(as_completed(seq))
             assert set(e.gather(seq2)) == {1, 2, 3, 4, 5}
+
+
+@pytest.mark.ipython
+def test_start_ipython(loop):
+    from jupyter_client import BlockingKernelClient
+    from ipykernel.kernelapp import IPKernelApp
+    from IPython.core.interactiveshell import InteractiveShell
+
+    with cluster(1) as (s, [a]):
+        with Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            info_dict = e.start_ipython()
+            info = first(info_dict.values())
+            key = info.pop('key')
+            kc = BlockingKernelClient(**info)
+            kc.session.key = key
+            kc.start_channels()
+            msg_id = kc.execute("worker")
+            reply = kc.get_shell_msg(timeout=10)
+            kc.stop_channels()
+
+@pytest.mark.ipython
+def test_start_ipython_magic(loop):
+    ip = mock.Mock()
+    with cluster() as (s, [a, b]):
+        with mock.patch('IPython.get_ipython', lambda : ip), Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            workers = list(e.ncores())[:2]
+            names = [ 'magic%i' % i for i in range(len(workers)) ]
+            e.start_ipython(workers, magic_names=names)
+    assert ip.register_magic_function.call_count == 4
+    expected = [
+        {'magic_kind': 'line', 'magic_name': 'magic0'},
+        {'magic_kind': 'cell', 'magic_name': 'magic0'},
+        {'magic_kind': 'line', 'magic_name': 'magic1'},
+        {'magic_kind': 'cell', 'magic_name': 'magic1'},
+    ]
+    call_kwargs_list = [ kwargs for (args, kwargs) in ip.register_magic_function.call_args_list ]
+    assert call_kwargs_list == expected
+
+
+@pytest.mark.ipython
+def test_start_ipython_qtconsole(loop):
+    Popen = mock.Mock()
+    with cluster() as (s, [a, b]):
+        with mock.patch('distributed._ipython_utils.Popen', Popen), Executor(('127.0.0.1', s['port']), loop=loop) as e:
+            worker = first(e.ncores())
+            e.start_ipython(worker, qtconsole=True)
+            e.start_ipython(worker, qtconsole=True, qtconsole_args=['--debug'])
+    assert Popen.call_count == 2
+    (cmd,), kwargs = Popen.call_args_list[0]
+    assert cmd[:3] == [ 'jupyter', 'qtconsole', '--existing' ]
+    (cmd,), kwargs = Popen.call_args_list[1]
+    assert cmd[-1:] == [ '--debug' ]
+
+
+@gen_cluster(ncores=[], executor=True, timeout=None)
+def test_stress_creation_and_deletion(e, s):
+    da = pytest.importorskip('dask.array')
+
+    x = da.random.random(size=(2000, 2000), chunks=(100, 100))
+    y = (x + 1).T + (x * 2) - x.mean(axis=1)
+
+    z = e.persist(y)
+
+    @gen.coroutine
+    def create_and_destroy_worker(delay):
+        start = time()
+        while time() < start + 10:
+            n = Nanny(s.ip, s.port, ncores=2, loop=s.loop)
+            n.start(0)
+
+            yield gen.sleep(delay)
+
+            yield n._close()
+            print("Killed nanny")
+
+    yield [create_and_destroy_worker(0.1 * i) for i in range(10)]
